@@ -56,6 +56,7 @@
 #'
 #' @examples
 #' if (bru_safe_inla() &&
+#'   bru_safe_sp() &&
 #'   require(ggplot2, quietly = TRUE) &&
 #'   require(ggpolypath, quietly = TRUE)) {
 #'   data(gorillas, package = "inlabru") # get the data
@@ -93,25 +94,26 @@ import.gorillas <- function() {
   data(gorillas, package = "spatstat.data", envir = environment())
 
   # Create SpatialPoints representing nest locations
+  requireNamespace("spatstat.geom")
   nests <- as.data.frame(gorillas)
   coordinates(nests) <- c("x", "y")
-  crs <- sp::CRS("+proj=utm +zone=32N +datum=WGS84") # from the Gorillas help file
-  crs_km <- sp::CRS("+proj=utm +zone=32N +datum=WGS84 +units=km")
+  crs <- sp::CRS("+proj=utm +zone=32 N +datum=WGS84") # from the Gorillas help file
+  crs_km <- sp::CRS("+proj=utm +zone=32 N +datum=WGS84 +units=km")
   proj4string(nests) <- crs
 
-  #' Turn the observation window into spatial polygon
+  # Turn the observation window into spatial polygon
   boundary <- spoly(as.data.frame(gorillas$window$bdry[[1]]),
     crs = crs
   )
 
-  #' Build mesh
-  bnd <- INLA::inla.sp2segment(boundary)
+  # Build mesh
+  bnd <- fm_as_inla_mesh_segment(boundary)
   mesh <- INLA::inla.mesh.2d(
     interior = bnd, max.edge = 222,
     crs = crs
-  ) # ! With higher max.edge we run into various INLA errors/warnings
+  ) # With higher max.edge we run into various INLA errors/warnings
 
-  #' Turn covariates int SpatialGridDataFrame
+  # Turn covariates int SpatialGridDataFrame
   gcov <- list()
   for (nm in names(gorillas.extra)) {
     gcov[[nm]] <- as(gorillas.extra[[nm]], "SpatialGridDataFrame")
@@ -123,12 +125,13 @@ import.gorillas <- function() {
     }
   }
 
-  #' Hack: change CRS units of the covariates to km
+  # Hack: Change CRS units of the covariates to km
   for (nm in names(gcov)) {
     ga <- attributes(gcov[[nm]])$grid
     attributes(ga)$cellcentre.offset <- attributes(ga)$cellcentre.offset / 1000
     attributes(ga)$cellsize <- attributes(ga)$cellsize / 1000
     attributes(gcov[[nm]])$grid <- ga
+    attributes(gcov[[nm]])$bbox <- attributes(gcov[[nm]])$bbox / 1000
     attributes(gcov[[nm]])$proj4string <- crs_km
   }
 
@@ -141,7 +144,9 @@ import.gorillas <- function() {
     gcov = gcov
   )
 
-  gorillas <- stransform(gorillas, crs_km)
+  gorillas$nests <- fm_transform(gorillas$nests, crs_km)
+  gorillas$mesh <- fm_transform(gorillas$mesh, crs_km)
+  gorillas$boundary <- fm_transform(gorillas$boundary, crs_km)
 
   # Create a plot sampling data set
   set.seed(121)
@@ -172,14 +177,41 @@ import.gorillas <- function() {
   proj4string(gorillas$plotsample$counts) <- crs
 
   # Extrapolate covariate
-  pxl <- pixels(gorillas$mesh, mask = FALSE, nx = 220, ny = 180)
-  for (k in seq_len(length(gorillas$gcov))) {
-    gorillas$gcov[[k]] <- sfill(gorillas$gcov[[k]], pxl)
+  pxl <- fm_pixels(gorillas$mesh,
+    mask = FALSE, nx = 220, ny = 180,
+    format = "sp"
+  )
+  pxl <- fm_transform(pxl, fm_crs(gorillas$gcov[[1]]))
+  for (k in names(gorillas$gcov)) {
+    NA_value <- gorillas$gcov[[k]][[1]][1]
+    is.na(NA_value) <- NA
+    pxl[[k]] <- NA_value
+    pxl[[k]] <- bru_fill_missing(gorillas$gcov[[k]], pxl, values = pxl[[k]])
   }
+  gorillas$gcov <- pxl
 
   return(gorillas)
 }
 
 
+#' @describeIn import.gorillas Convert gorillas to `sf` and `terra` format
+import.gorillas.sf <- function() {
+  gorillas <- NULL
+  data(gorillas, package = "inlabru", envir = environment())
+
+  gorillas_sf <- list()
+  gorillas_sf$nests <- sf::st_as_sf(gorillas$nests)
+  gorillas_sf$mesh <- gorillas$mesh
+  gorillas_sf$boundary <- sf::st_as_sf(gorillas$boundary)
+  gorillas_sf$gcov <- terra::rast(gorillas$gcov[[1]])
+  for (k in seq_len(length(gorillas$gcov) - 1L) + 1L) {
+    terra::add(gorillas_sf$gcov) <- terra::rast(gorillas$gcov[[k]])
+  }
+  gorillas_sf$plotsample <- lapply(gorillas$plotsample, sf::st_as_sf)
+
+  gorillas_sf
+}
+
+
 # gorillas <- import.gorillas()
 # use_data(gorillas)
