@@ -4,7 +4,7 @@
 #' @description This a version of the `mexdolphins` dataset from the package `dsm`, reformatted
 #' as point process data for use with `inlabru`. The data are from a combination of several NOAA
 #' shipboard surveys conducted on pan-tropical spotted dolphins in the Gulf of Mexico. 47 observations
-#' of groups of dolphins wre detected. The group size was recorded, as well as the Beaufort sea state at
+#' of groups of dolphins were detected. The group size was recorded, as well as the Beaufort sea state at
 #' the time of the observation. Transect width is 16 km, i.e. maximal detection
 #' distance 8 km (transect half-width 8 km).
 #'
@@ -207,4 +207,120 @@ import.mexdolphin <- function() {
   mexdolphin
 }
 
+#' @describeIn import.mexdolphin Import mexdolphin data as `sf`
+import.mexdolphin.sf <- function() {
+  envir <- new.env()
+  data("mexdolphins", package = "dsm", envir = envir)
+  mexdolphins <- as.list(envir)
+  data.p4s <- "+proj=lcc +lat_1=20 +lat_2=60 +lat_0=40 +lon_0=-96 +x_0=0 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=m +no_defs"
+  data_crs <- fm_crs(data.p4s)
+
+  dset <- import.dsmdata(mexdolphins, covar.col = 8)
+  dset$mesh$crs <- fm_CRS(data_crs)
+  mexdolphin <- as.spatial.dsdata(dset, cnames = c("x", "y"), crs = fm_CRS(data_crs))
+  mexdolphin$points <- sf::st_as_sf(mexdolphin$points)
+  mexdolphin$samplers <- sf::st_as_sf(mexdolphin$sampler)
+
+  # Target CRS
+  target.p4s <- "+proj=lcc +lat_1=20 +lat_2=60 +lat_0=40 +lon_0=-96 +x_0=0 +y_0=0 +ellps=GRS80 +datum=NAD83 +units=km +no_defs +towgs84=0,0,0"
+  target_crs <- fm_crs(target.p4s)
+
+  # Units to km
+  mexdolphin$points <- fm_transform(mexdolphin$points, crs = target_crs)
+  mexdolphin$samplers <- fm_transform(mexdolphin$samplers, crs = target_crs)
+  mexdolphin$points$distance <- mexdolphin$points$distance / 1000
+  mexdolphin$points$Effort <- mexdolphin$points$Effort / 1000
+  mexdolphin$points$mid.x <- mexdolphin$points$mid.x / 1000
+  mexdolphin$points$mid.y <- mexdolphin$points$mid.y / 1000
+  mexdolphin$points$start.x <- mexdolphin$points$start.x / 1000
+  mexdolphin$points$start.y <- mexdolphin$points$start.y / 1000
+  mexdolphin$points$end.x <- mexdolphin$points$end.x / 1000
+  mexdolphin$points$end.y <- mexdolphin$points$end.y / 1000
+  mexdolphin$samplers$mid.x <- mexdolphin$samplers$mid.x / 1000
+  mexdolphin$samplers$mid.y <- mexdolphin$samplers$mid.y / 1000
+  mexdolphin$samplers$Effort <- mexdolphin$samplers$Effort / 1000
+  mexdolphin$mesh <- fm_transform(mexdolphin$mesh, crs = target_crs)
+
+
+  # Remove all-NA columns such as "distance" from samplers, since they may
+  # interfere with normal usage.
+  all_na <- vapply(
+    names(mexdolphin$samplers),
+    function(nm) {
+      all(is.na(mexdolphin$samplers[[nm]]))
+    },
+    TRUE
+  )
+  mexdolphin$samplers <- mexdolphin$samplers[!all_na]
+
+  ##### Prediction polygon
+  polyloc <- as.data.frame(
+    mexdolphin$mesh$loc[mexdolphin$mesh$segm$int$idx[, 1], c(1, 2)]
+  )
+  colnames(polyloc) <- c("x", "y")
+  po <- sp::Polygon(polyloc, hole = FALSE)
+  pos <- sp::Polygons(list(po), ID = "c")
+  predpoly <- sp::SpatialPolygons(list(pos), proj4string = fm_CRS(target_crs))
+  df <- data.frame(weight = 1)
+  rownames(df) <- "c"
+  predpolyd <- sp::SpatialPolygonsDataFrame(predpoly, data = df)
+  # plot(predpolyd)
+  mexdolphin$ppoly <- sf::st_as_sf(predpolyd)
+
+  ##### Simulate a whole population #####
+  distance <- seq(0, 8, length.out = 20)
+  matern <- INLA::inla.spde2.pcmatern(
+    mexdolphin$mesh,
+    prior.range = c(50, 0.01),
+    prior.sigma = c(2, 0.01)
+  )
+  cmps <-
+    geometry + distance ~
+    Intercept(1) +
+    df.lsigma(1) +
+    spat(geometry, model = matern)
+  pred <- ~ log(1 - exp(-(distance / exp(df.lsigma))^-1)) + spat + Intercept + log(2)
+  r <- lgcp(
+    data = mexdolphin$points,
+    samplers = cbind(mexdolphin$samplers, data.frame(weight = 1)),
+    components = cmps,
+    formula = pred,
+    domain = list(
+      geometry = mexdolphin$mesh,
+      distance = distance
+    ),
+    options = list(
+      bru_verbose = 3,
+      control.inla = list(int.strategy = "eb")
+    )
+  )
+  llambda <- r$summary.random$spat$mean + r$summary.fixed["Intercept", "mean"]
+
+  smexdolphin <- mexdolphin
+  smexdolphin$llambda <- llambda
+  mexdolphin$lambda <- exp(llambda)
+
+  set.seed(1234L)
+  pts <- sample.lgcp(
+    mexdolphin$mesh,
+    loglambda = llambda,
+    samplers = mexdolphin$ppoly
+  )
+
+  mexdolphin$simulated <- sf::st_as_sf(pts)
+  mexdolphin$simulated$size <- 1
+  #  sf::st_crs(mexdolphin$simulated) <- fm_CRS(target_crs)
+
+  #### Depth covariate #####
+  depth <- sf::st_as_sf(
+    mexdolphins$preddata[, c("x", "y", "depth"), drop = FALSE],
+    coords = c("x", "y"),
+    crs = fm_crs(data_crs)
+  )
+  mexdolphin$depth <- fm_transform(depth, target_crs)
+
+  # return
+  mexdolphin
+}
+
 # use_data(mexdolphin, compress = "xz")
