@@ -2,7 +2,7 @@
 #' @rdname fm_as
 #' @param ... Arguments passed on to other methods
 #' @export
-fm_as_sfc <- function(x) {
+fm_as_sfc <- function(x, ...) {
   UseMethod("fm_as_sfc")
 }
 
@@ -24,22 +24,39 @@ fm_as_inla_mesh <- function(...) {
 #' @aliases fm_as_sfc fm_as_sfc.inla.mesh
 #'
 #' @param x An object to be coerced/transformed/converted into another class
-#' @returns * `fm_as_sfc`: An `sfc_MULTIPOLYGON` object
+#' @param multi logical; if `TRUE`, attempt to a `sfc_MULTIPOLYGON`, otherwise
+#' a set of `sfc_POLYGON`. Default `FALSE`
+#' @returns * `fm_as_sfc`: An `sfc_MULTIPOLYGON` or `sfc_POLYGON` object
 #' @exportS3Method fm_as_sfc inla.mesh
 #' @export
-fm_as_sfc.inla.mesh <- function(x, ...) {
+fm_as_sfc.inla.mesh <- function(x, ..., multi = FALSE) {
   stopifnot(inherits(x, "inla.mesh"))
-  geom <- sf::st_geometry(
-    sf::st_multipolygon(
+  if (multi) {
+    geom <- sf::st_sfc(
+      sf::st_multipolygon(
+        lapply(
+          seq_len(nrow(x$graph$tv)),
+          function(k) {
+            list(x$loc[x$graph$tv[k, c(1, 2, 3, 1)], , drop = FALSE])
+          }
+        ),
+        dim = "XYZ"
+      ),
+      check_ring_dir = TRUE
+    )
+  } else {
+    geom <- sf::st_sfc(
       lapply(
         seq_len(nrow(x$graph$tv)),
         function(k) {
-          list(x$loc[x$graph$tv[k, c(1, 2, 3, 1)], , drop = FALSE])
+          sf::st_polygon(
+            list(x$loc[x$graph$tv[k, c(1, 2, 3, 1)], , drop = FALSE]),
+            dim = "XYZ"
+          )
         }
-      ),
-      dim = "XYZ"
+      )
     )
-  )
+  }
   sf::st_crs(geom) <- fm_crs(x$crs)
   geom
 }
@@ -56,6 +73,9 @@ fm_as_inla_mesh.sfc_MULTIPOLYGON <- function(x, ...) {
       immediate. = TRUE
     )
   }
+  # Ensure correct CCW ring orientation; sf doesn't take into account
+  # that geos has CW as canonical orientation
+  x <- sf::st_sfc(x, check_ring_dir = TRUE)
   tv <- matrix(seq_len(3 * length(x[[1]])), length(x[[1]]), 3, byrow = TRUE)
   loc <- do.call(
     rbind,
@@ -78,6 +98,36 @@ fm_as_inla_mesh.sfc_MULTIPOLYGON <- function(x, ...) {
   mesh
 }
 
+#' @rdname fm_as
+#' @aliases fm_as_inla_mesh fm_as_inla_mesh.sfc_POLYGON
+#'
+#' @export
+fm_as_inla_mesh.sfc_POLYGON <- function(x, ...) {
+  # Ensure correct CCW ring orientation; sf doesn't take into account
+  # that geos has CW as canonical orientation
+  sfc <- sf::st_sfc(x, check_ring_dir = TRUE)
+  tv <- matrix(seq_len(3 * NROW(x)), NROW(x), 3, byrow = TRUE)
+  loc <- do.call(
+    rbind,
+    lapply(
+      x,
+      function(xx) {
+        if ((length(xx) > 1) ||
+          (nrow(xx[[1]]) > 4)) {
+          stop("Invalid geometry; non-triangle detected.")
+        }
+        xx[[1]][1:3, , drop = FALSE]
+      }
+    )
+  )
+  crs <- fm_CRS(sf::st_crs(x))
+  mesh <- INLA::inla.mesh.create(
+    loc = loc, tv = tv, ...,
+    crs = crs
+  )
+  mesh
+}
+
 
 
 
@@ -180,7 +230,9 @@ fm_as_inla_mesh_segment.sfc_LINESTRING <-
 #' @export
 fm_as_inla_mesh_segment.sfc_POLYGON <-
   function(x, join = TRUE, grp = NULL, ...) {
-    sfc <- x
+    # Ensure correct CCW ring orientation; sf doesn't take into account
+    # that geos has CW as canonical orientation
+    sfc <- sf::st_sfc(x, check_ring_dir = TRUE)
     crs <- sf::st_crs(sfc)
     crs <- fm_CRS(crs) # required for INLA::inla.mesh.segment
 
@@ -191,15 +243,66 @@ fm_as_inla_mesh_segment.sfc_POLYGON <-
     for (k in seq_len(length(sfc))) {
       loc <- sf::st_coordinates(sfc[k])
       coord_names <- intersect(c("X", "Y", "Z"), colnames(loc))
-      Linfo <- loc[, "L2", drop = TRUE]
+      L1info <- loc[, "L1", drop = TRUE]
+      L2info <- loc[, "L2", drop = TRUE]
       loc <- unname(loc[, coord_names, drop = FALSE])
       # If winding directions are correct, all info is already available
       # For 3D, cannot check winding, so must assume correct.
       segm_k <-
         lapply(
-          unique(Linfo),
+          unique(L1info),
           function(i) {
-            subset <- which(Linfo == i)
+            subset <- which(L1info == i)
+            # sfc_POLYGON repeats the initial point within each L1
+            n <- length(subset) - 1
+            subset <- subset[-(n + 1)]
+            idx <- c(seq_len(n), 1L)
+            INLA::inla.mesh.segment(
+              loc = loc[subset, , drop = FALSE],
+              idx = idx,
+              grp = grp[k],
+              is.bnd = TRUE,
+              crs = crs
+            )
+          }
+        )
+      segm[[k]] <- fm_internal_sp2segment_join(segm_k)
+    }
+
+    if (join) {
+      segm <- fm_internal_sp2segment_join(segm, grp = grp)
+    }
+    segm
+  }
+
+#' @rdname fm_as
+#' @export
+fm_as_inla_mesh_segment.sfc_MULTIPOLYGON <-
+  function(x, join = TRUE, grp = NULL, ...) {
+    # Ensure correct CCW ring orientation; sf doesn't take into account
+    # that geos has CW as canonical orientation
+    sfc <- sf::st_sfc(x, check_ring_dir = TRUE)
+    crs <- sf::st_crs(sfc)
+    crs <- fm_CRS(crs) # required for INLA::inla.mesh.segment
+
+    segm <- list()
+    if (is.null(grp)) {
+      grp <- seq_len(length(sfc))
+    }
+    for (k in seq_len(length(sfc))) {
+      loc <- sf::st_coordinates(sfc[k])
+      coord_names <- intersect(c("X", "Y", "Z"), colnames(loc))
+      Linfo <- loc[, c("L1", "L2"), drop = FALSE]
+      loc <- unname(loc[, coord_names, drop = FALSE])
+      # If winding directions are correct, all info is already available
+      # For 3D, cannot check winding, so must assume correct.
+      uniqueLinfo <- unique(Linfo)
+      segm_k <-
+        lapply(
+          seq_len(nrow(uniqueLinfo)),
+          function(i) {
+            subset <- which((Linfo[, 1] == uniqueLinfo[i, 1]) &
+              (Linfo[, 2] == uniqueLinfo[i, 2]))
             # sfc_POLYGON repeats the initial point
             n <- length(subset) - 1
             subset <- subset[-(n + 1)]
@@ -222,12 +325,32 @@ fm_as_inla_mesh_segment.sfc_POLYGON <-
     segm
   }
 
+#' @rdname fm_as
+#' @export
+fm_as_inla_mesh_segment.sfc_GEOMETRY <-
+  function(x, grp = NULL, join = TRUE, ...) {
+    if (is.null(grp)) {
+      grp <- seq_len(length(x))
+    }
+    segm <-
+      lapply(
+        seq_along(x),
+        function(k) {
+          fm_as_inla_mesh_segment(x[k], grp = grp[[k]], join = join, ...)
+        }
+      )
+    if (join) {
+      segm <- fm_internal_sp2segment_join(segm, grp = grp)
+    }
+    segm
+  }
+
 #' @rdname fm_as
 #' @export
 fm_as_inla_mesh_segment.sf <-
-  function(x, reverse = FALSE, grp = NULL, is.bnd = TRUE, ...) {
+  function(x, ...) {
     sfc <- sf::st_geometry(x)
-    fm_as_inla_mesh_segment(sfc)
+    fm_as_inla_mesh_segment(sfc, ...)
   }
 
 #' @rdname fm_as
@@ -235,5 +358,5 @@ fm_as_inla_mesh_segment.sf <-
 fm_as_inla_mesh.sf <-
   function(x, ...) {
     sfc <- sf::st_geometry(x)
-    fm_as_inla_mesh(sfc)
+    fm_as_inla_mesh(sfc, ...)
   }
