@@ -214,7 +214,10 @@ spde.posterior <- function(result, name, what = "range") {
     }
 
 
-    df <- data.frame(x = x, median = out$median, q0.025 = out$lower, q0.975 = out$upper)
+    df <- data.frame(
+      x = x, q0.5 = out$median, q0.025 = out$lower, q0.975 = out$upper,
+      median = out$median
+    )
     attr(df, "type") <- "1d"
     attr(df, "misc") <- list(dims = "x", predictor = c("distance", ylab))
     class(df) <- list("prediction", "data.frame")
