@@ -1,5 +1,7 @@
 \donttest{
-if (bru_safe_inla() && require(ggplot2, quietly = TRUE)) {
+if (bru_safe_inla() &&
+    require(ggplot2, quietly = TRUE) &&
+    bru_safe_sp()) {
 
   # Load Gorilla data
 
@@ -30,7 +32,7 @@ if (bru_safe_inla() && require(ggplot2, quietly = TRUE)) {
 
   # Predict SPDE and vegetation at the mesh vertex locations
 
-  vrt <- vertices.inla.mesh(gorillas$mesh)
+  vrt <- fm_vertices(gorillas$mesh, format = "sp")
   pred <- predict(
     fit,
     vrt,
@@ -119,7 +121,7 @@ if (bru_safe_inla() && require(ggplot2, quietly = TRUE)) {
   # Variance and correlation integrated over space
 
   vm.int <- devel.cvmeasure(pred$joint, pred$field, pred$veg,
-    samplers = ipoints(gorillas$boundary, gorillas$mesh),
+    samplers = fm_int(gorillas$mesh, gorillas$boundary),
     mesh = gorillas$mesh
   )
   vm.int
