@@ -1,5 +1,6 @@
 \donttest{
-if (bru_safe_inla(multicore = FALSE)) {
+if (bru_safe_inla(multicore = FALSE) &&
+    require(ggplot2, quietly = TRUE)) {
 
   # Generate some data
   input.df <- data.frame(x = cos(1:10))
@@ -24,13 +25,13 @@ if (bru_safe_inla(multicore = FALSE)) {
   xpost
 
   # For a single variable like 'x' the default plotting method invoked by gg() will
-  # show these statisics in a fashion similar to a box plot:
+  # show these statistics in a fashion similar to a box plot:
 
   ggplot() +
     gg(xpost)
 
 
-  # The predict function can also be used to simulatenneously estimate posteriors
+  # The predict function can also be used to simultaneously estimate posteriors
   # of multiple variables:
 
   xipost <- predict(fit, formula = ~ data.frame(post = c(x, Intercept)))
@@ -42,4 +43,4 @@ if (bru_safe_inla(multicore = FALSE)) {
   ggplot() +
     gg(xipost, bar = TRUE)
 }
-}
\ No newline at end of file
+}
