@@ -1,7 +1,8 @@
 \dontrun{
 if (bru_safe_inla() &&
   require("rgl", quietly = TRUE) &&
-  require("sphereplot", quietly = TRUE)) {
+  require("sphereplot", quietly = TRUE) &&
+  bru_safe_sp()) {
 
   # Load pantropoical dolphin data
 
